<?php
namespace PME\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

class Lista extends \PME\Services\TeknisaCrud {

    protected $dataSourceName = "lista";
    protected $entityManager;
    protected $novoCodigo;


    public function __construct(
        $crudService, $parameterBag,
        $entityManager,
        $novoCodigo
    ) {
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        parent::__construct($crudService, $parameterBag);
    }

    public function geraLista($cdlista, $nrorg){
        $rows = array_unique(explode("_", $cdlista));

        $nrlista = $this->novoCodigo->getCodeByTableName($this->dataSourceName, $nrorg);

        $registros = array();
        foreach ($rows as $row) {
            if(!empty($row)){
                $row = array(
                    "NRLISTA"  => $nrlista,
                    "CDLISTA"  => $row,
                    "__is_new" => true,
                    "NRORG"    => $nrorg
                );
                array_push($registros, $row);
            }
        }
        
        $dataSet = new DataSet($this->dataSourceName, $registros);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
        $newRows = [];
        if(!empty($rowsToSave)){
            foreach($rowsToSave as $row){
                if(!\Zeedhi\Framework\Util\Functions::arrayKeyExists('DTULTATULIST', $row)) $row['DTULTATULIST'] = $row['DTULTATU'];
                array_push($newRows, $row);
            }
        }
        
        $this->crudService->save($this->dataSourceName, $newRows);
        
        return $nrlista;
    }
    
    public function insereListaProduto(DTO\Request $request, DTO\Response $response){
        $parametros = $request->getRow();
        $cdlista = $parametros['CDLISTA'];
        $nrorg = $parametros['NRORG'];
        
        $nrlista = $this->geraLista($cdlista, $nrorg);
        
        $response->addDataSet(new DataSet ('LISTAPRODUTO', array('LISTA' => $nrlista)));
    }
    
    public function insereListaServico(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $nrlistaServico = $this->geraListaServico($row['FILISERV'], $row['NRORG']);
        $response->addDataSet(
            new DataSet ('NRLISTASERVICO', array('NRLISTASERVICO' => $nrlistaServico))
        );
    }
    
    public function geraListaServico($filiserv, $nrorg){
        $servicosFiliais = explode("_",$filiserv);
        
        $nrlistaServico = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $nrorg);
        
        $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";
        
        foreach ($servicosFiliais as $servicoFilial) {
            if(!empty($servicoFilial)){
                $servicoFilial = $this->infoServico($servicoFilial);
                if(!empty($servicoFilial)){
                    $stmt = $this->entityManager->getConnection()->prepare($sql);
                    $stmt->execute(
                        array(
                            "NRLISTA" => $nrlistaServico,
                            "CDFILIAL" => $servicoFilial['CDFILIAL'],
                            "NRORG" => $nrorg,
                            "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
                            "NMSERVICO" => $servicoFilial['NMSERVICO'],
                            "CDGRUPSERV" => $servicoFilial['CDGRUPSERV']
                        )
                    );   
                }
            }
            
        }
        
        return $nrlistaServico;
    }
    
    private function infoServico($row){
        $cdfilial = substr($row,0,4);
        $cdservfili = substr($row,4,5);
        $sql = "SELECT S.CDFILIAL, S.CDSERVFILI,S.NMSERVICO, G.CDGRUPSERV FROM SERVICO S
                    INNER JOIN GRUPSERV G ON G.CDGRUPSERV = S.CDGRUPSERV
                WHERE S.CDFILIAL =:CDFILIAL AND S.CDSERVFILI =:CDSERVFILI";
        $stmt = $this->entityManager->getConnection()->prepare($sql);
        $stmt->execute(
            array(
                "CDFILIAL"   => $cdfilial,
                "CDSERVFILI"  => $cdservfili
            )
        );
        $result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        if(!empty($result)) $servico = $result[0];
        else $servico = null;
        return $servico;
    }
	
	public function geraListaTipoCard($tpCard, $nrorg) {
    $nrlista = $this->novoCodigo->getCodeByTableName('LISTA_TIPOCARD', $nrorg);
    $sql = "INSERT INTO LISTA_TIPOCARD (NRLISTA, CDFILCENTPME, CDTIPOCARD, NMTIPOCARD, NRORG) VALUES (:NRLISTA, :CDFILCENTPME, :CDTIPOCARD, :NMTIPOCARD, :NRORG)";
    
    foreach ($tpCard as $card) {
      $stmt = $this->entityManager->getConnection()->prepare($sql);
      $stmt->execute(
          array(
            "NRLISTA"      => $nrlista,
            "CDFILCENTPME" => $card['CDFILCENTPME'],
            "CDTIPOCARD"   => $card["CDTIPOCARD"],
            "NMTIPOCARD"   => $card['NMTIPOCARD'],
            "NRORG"        => $nrorg
          )
      );
    }
    
    return $nrlista;
  }
  
  public function deleteLista(DTO\Request $request, DTO\Response $response){
    try {
        $rows = $request->getRow();
        $this->remove($rows['NRLISTASERV']);
        $response->addDataSet(new DataSet ('DELETED', array(true)));
    } catch (\Exception $e) {
        $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function remove($nrlista){
    $sql = 'DELETE FROM LISTA WHERE NRLISTA = :P_NRLISTA';
    $params = array(
        "P_NRLISTA" => $nrlista
    );
    
    $std = $this->entityManager->getConnection()->prepare($sql);
    $std->execute($params); 
  }
}